
package controllers.interceptors;

import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.juliac.Juliac;
import org.objectweb.fractal.juliac.desc.MembraneDesc;
import org.objectweb.fractal.juliac.proxy.AbstractInterceptorSourceCodeGenerator;
import org.objectweb.fractal.juliac.proxy.SimpleSourceCodeGeneratorMode;
import org.objectweb.fractal.juliac.ucf.UnifiedClass;
import org.objectweb.fractal.juliac.ucf.UnifiedMethod;
import org.objectweb.fractal.juliac.visit.FileSourceCodeVisitor;
import org.objectweb.fractal.juliac.visit.MethodSourceCodeVisitor;
import org.omg.CORBA.ORB;

/**
 * This class generates the source code of lifecycle interceptors.
 *
 * @author Lionel Seinturier <Lionel.Seinturier@lifl.fr>
 */
public class SoleilSkeletonSourceCodeGenerator extends SoleilAbstractConnectorSourceCodeGenerator {
        
    public SoleilSkeletonSourceCodeGenerator() {
        super();
    }
    
    public SoleilSkeletonSourceCodeGenerator(
        InterfaceType it, MembraneDesc membraneDesc, boolean mergeable, Juliac jc) {
    	        
        super(it, membraneDesc, mergeable, jc );        
    }
    
    /*
     * CODE GENERATION PART 
     */
    
    
    
    @Override
    public String getClassNameSuffix() {
    	return super.getClassNameSuffix() + "Skeleton";
    }  
    
    @Override
    public void generateMethodInitFcController(MethodSourceCodeVisitor mv, UnifiedClass proxycl) {
    	mv.visitln("try {");
    	mv.visitln("ORB orb = ORB.init(new String[] {}, null);");
    	mv.visitln("POA rootPOA = POAHelper.narrow(orb.resolve_initial_references(\"RootPOA\"));");
    	mv.visitln("rootPOA.the_POAManager().activate();");

    	mv.visitln("org.omg.CORBA.Object obj = rootPOA.servant_to_reference(new ServicePOATie(this));");
    	mv.visitln("String ior = orb.object_to_string(obj);");

    	mv.visitln("System.out.println(\"[server] Running Hello World Example... \");");
    	mv.visitln("System.out.println(\"[Server] \" + ior );");
    	
    	mv.visitln("BufferedWriter bw = new BufferedWriter( new FileWriter(\"ior.txt\") );");
    	mv.visitln("bw.write(ior);");
    	mv.visitln("bw.close();");
    	mv.visitln("} catch (Exception e) { ");
    	mv.visitln(" e.printStackTrace(); throw new org.objectweb.fractal.api.factory.InstantiationException(e.getMessage()); }");
    }
    
    @Override
    public void generateImports(FileSourceCodeVisitor fv) {

    	fv.visitImport("java.io.*");
    	fv.visitImport("org.omg.CORBA.ORB");
    	//fv.visitImport("javax.realtime.*");
    	fv.visitImport("org.omg.PortableServer.*");
    }
    
	@Override
	public void generateProxyMethodBodyBeforeCode(MethodSourceCodeVisitor mv,
			UnifiedClass proxycl, UnifiedMethod proxym) {		
		super.generateProxyMethodBodyBeforeCode(mv, proxycl, proxym);
		
		mv.visitln("System.out.println(\"[server] SoleilSkeleton: Pre call\");");
	}
	
	/**
	 * Just intercept on provided interfaces.
	 */
	@Override
	protected SimpleSourceCodeGeneratorMode getMode() {
		return SimpleSourceCodeGeneratorMode.IN;
	}
}
