
package controllers.interceptors;

import java.io.BufferedReader;
import java.io.File;
import java.lang.reflect.Modifier;

import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.juliac.Juliac;
import org.objectweb.fractal.juliac.desc.MembraneDesc;
import org.objectweb.fractal.juliac.proxy.AbstractInterceptorSourceCodeGenerator;
import org.objectweb.fractal.juliac.proxy.SimpleSourceCodeGeneratorMode;
import org.objectweb.fractal.juliac.ucf.UnifiedClass;
import org.objectweb.fractal.juliac.ucf.UnifiedMethod;
import org.objectweb.fractal.juliac.visit.ClassSourceCodeVisitor;
import org.objectweb.fractal.juliac.visit.FileSourceCodeVisitor;
import org.objectweb.fractal.juliac.visit.MethodSourceCodeVisitor;


/**
 * This class generates the source code of lifecycle interceptors.
 *
 * @author Lionel Seinturier <Lionel.Seinturier@lifl.fr>
 */
public class SoleilStubSourceCodeGenerator extends SoleilAbstractConnectorSourceCodeGenerator {
        
    public SoleilStubSourceCodeGenerator() {
        super();
    }
    
    public SoleilStubSourceCodeGenerator(
        InterfaceType it, MembraneDesc membraneDesc, boolean mergeable, Juliac jc) {
    	        
        super(it, membraneDesc, mergeable, jc );        
    }
    
    @Override
    public String getClassNameSuffix() {
       	return super.getClassNameSuffix() + "Stub";
    }
    
    // accept just required ifaces
    @Override
	protected SimpleSourceCodeGeneratorMode getMode() {
		return SimpleSourceCodeGeneratorMode.OUT;
    }
    
    
    /*
     * CODE GENERATION PART 
     */
    
    @Override
    public void generateImports(FileSourceCodeVisitor fv) {
    	super.generateImports(fv);
    	fv.visitImport("org.omg.CORBA.ORB");
    	fv.visitImport("java.io.*");
    	fv.visitImport("example.hw.*");
    }
    
    @Override
    public void generateMethodInitFcController(MethodSourceCodeVisitor mv, UnifiedClass proxycl) {
    	    	
    	mv.visitln("ORB orb = ORB.init(new String[] {}, null);");
    }
    
    @Override
    public void generateFields(ClassSourceCodeVisitor cv, UnifiedClass proxycl) {
    	String remoteObjectIface = it.getFcItfSignature();
    	
    	cv.visitField(Modifier.PUBLIC, remoteObjectIface , "_remoteRef", null);
    	cv.visitField(Modifier.PUBLIC, "ORB" , "_orb", null);
    }
    
	@Override
	public void generateProxyMethodBodyBeforeCode(MethodSourceCodeVisitor mv,
			UnifiedClass proxycl, UnifiedMethod proxym) {		
		super.generateProxyMethodBodyBeforeCode(mv, proxycl, proxym);
		mv.visitln("System.out.println(\"SoleilStub: Pre call\");");    	
		mv.visitln("if (_remoteRef==null) {");
		mv.visitln("try {");
		mv.visitln("String ior = \"\";");
		mv.visitln("File iorFile = new File(\"ior.txt\");");		
		mv.visitln("BufferedReader br = new BufferedReader( new FileReader(iorFile) );");
		mv.visitln("ior = br.readLine();");
		mv.visitln("System.out.println(\"[client] IOR: \" + ior);");
		mv.visitln("org.omg.CORBA.Object object = _orb.string_to_object(ior);");
		mv.visitln("_remoteRef = ("+it.getFcItfSignature()+")" + getClassName(it.getFcItfSignature())+"Helper.unchecked_narrow(object);");
		mv.visitln("_impl = _remoteRef;");
		mv.visitln("} catch (Exception e) { e.printStackTrace(); }");
		mv.visitln("} // end of if");				
	}	
}
